//////////////////////////////////////
// test program for thermal camera
/////////////////////////////////////

#include <queue>

#define V4L2_BUFFERS_NUM    4

#define INFO(fmt, ...) \
    if (ctx->enable_verbose) \
        printf("INFO: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__);

#define WARN(fmt, ...) \
        printf("WARN: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__);

#define CHECK_ERROR(cond, label, fmt, ...) \
    if (!cond) { \
        error = 1; \
        printf("ERROR: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__); \
        goto label; \
    }

#define ERROR_RETURN(fmt, ...) \
    do { \
        printf("ERROR: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__); \
        return false; \
    } while(0)

typedef struct
{
    // Hold the user accessible pointer
    unsigned char * start;
    // Hold the memory length
    unsigned int size;
    // Hold the file descriptor
    int fd;
    // Hold index for camera queue
    int index;
} nv_buffer;

typedef struct
{
    // Hold the user accessible pointer
    unsigned char * start;
    // Hold the memory length
    unsigned int totalsize;
    unsigned int size[MAX_PLANES];
    unsigned int pitch[MAX_PLANES];
    // Hold the file descriptor
    int fd;
} yuv_buffer;

typedef struct
{
    // camera v4l2 context
    const char * cam_devname;
    char cam_file[16];
    int cam_fd;
    unsigned int cam_pixfmt;
    unsigned int cam_w;
    unsigned int cam_h;
    unsigned int frame;
    unsigned int save_n_frame;
    bool got_error;

    // Global buffer pointer
    nv_buffer * g_buff;
    yuv_buffer o_buff;

    // Used to do synchronization between camera and consumer
    std::queue < nv_buffer * > *output_plane_buf_queue;
    pthread_mutex_t queue_lock;
    pthread_cond_t queue_cond;

    // EGL renderer
    NvEglRenderer *renderer;
    int fps;

    // CUDA processing
    bool enable_cuda;
    CUcontext cuContext;
    EGLDisplay egl_display;
    EGLImageKHR egl_image;
    EGLImageKHR cam_image;

    // Verbose option
    bool enable_verbose;

} context_t;
