/////////////////////////////////////////
//   cuda module for Y to YUV conversion
////////////////////////////////////////
#include <stdio.h>

#include "y2yuv.h"

__global__ void gpuConvertY2YUV_kernel(unsigned char * src, unsigned char * ydst, unsigned char * udst, unsigned char * vdst, int width, int height, int llength, int ypitch, int uvpitch)
{
    int x = blockIdx.x * blockDim.x + threadIdx.x;
    int y = blockIdx.y * blockDim.y + threadIdx.y;
    int pindex = y*llength + x;
    int xp, yp;
    unsigned char pval;
	
    // copy y 
    if ((x < width) && (y < height))
    {
        pval = src[pindex];
        ydst[y*ypitch+x] = pval;
	
        //fill u and v 
        yp = y/2;
        xp = x/2;
        if (2*(threadIdx.x/2) == threadIdx.x)
        {
            pval += src[pindex+1];
            if (threadIdx.y == 0)
            {
               pval = (pval + src[pindex+llength] + src[pindex+llength+1])/4;
               if (pval <= 100)
                 vdst[yp*uvpitch+xp] = 128 + pval/4;                 
               else
                 vdst[yp*uvpitch+xp] = 128 + (250 - pval)/6;
             } 
            else  //i.e. threadIndx.y == 1
            {
               pval = (pval + src[pindex-llength] + src[pindex-llength+1])/4;
               if (pval <= 200)
                  udst[yp*uvpitch+xp] = 128 - pval/8;
               else
                  udst[yp*uvpitch+xp] = 128 - (250 - pval)/2;
            }
        } 
    }
	
}

int gpuConvertY2YUV(CUdeviceptr src, CUdeviceptr yArray, CUdeviceptr uArray, CUdeviceptr vArray, int w, int h, int ll, int ypitch, int uvpitch)
{

    unsigned char * sData = (unsigned char *) src;
    unsigned char * yData = (unsigned char *) yArray;
    unsigned char * uData = (unsigned char *) uArray;
    unsigned char * vData = (unsigned char *) vArray;

    dim3 dimBlock(128,2);
    dim3 dimGrid( (w + dimBlock.x-1)/dimBlock.x, (h + dimBlock.y-1)/dimBlock.y);
	
    gpuConvertY2YUV_kernel<<<dimGrid, dimBlock>>>(sData, yData, uData, vData, w, h, ll, ypitch, uvpitch);
    cudaError_t errsts = cudaGetLastError();
    if (errsts != cudaSuccess)
    {
        fprintf(stderr, "Failed to move frame data to global array %d\n", errsts);
        exit(errsts);
    }

    return errsts;
	
}


